package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.activiti.service.ActivitiService;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBProjectExchange;
import com.qmth.sop.business.entity.TFCustomFlowEntity;
import com.qmth.sop.business.mapper.TBProjectExchangeMapper;
import com.qmth.sop.business.service.TBProjectExchangeService;
import com.qmth.sop.business.service.TFCustomFlowEntityService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.DelayWarnTypeEnum;
import com.qmth.sop.common.enums.FlowApprovePassEnum;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.enums.ProjectExchangeFlowStatusEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 项目变更计划表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBProjectExchangeServiceImpl extends ServiceImpl<TBProjectExchangeMapper, TBProjectExchange> implements TBProjectExchangeService {

    @Resource
    ActivitiService activitiService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    /**
     * 新增项目变更计划申请
     *
     * @param tbProjectExchange
     * @return
     * @throws InterruptedException
     */
    @Override
    @Transactional
    public Boolean saveProjectExchange(TBProjectExchange tbProjectExchange) throws InterruptedException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, Object> map = activitiService.taskApprove(new FlowApproveParam(tbProjectExchange.getFlowDeploymentId(), FlowApprovePassEnum.START, !CollectionUtils.isEmpty(tbProjectExchange.getApproveUserIds()) ? tbProjectExchange.getApproveUserIds() : Arrays.asList(String.valueOf(sysUser.getId())), tbProjectExchange.getCrmNo()));
        TFCustomFlowEntity tfCustomFlowEntity = (TFCustomFlowEntity) map.get(SystemConstant.FLOW_ENTITY);
        tbProjectExchange.setCode(tfCustomFlowEntity.getCode(), sysUser.getId());
        tfCustomFlowEntity.setObjId(tbProjectExchange.getId());
        tfCustomFlowEntityService.updateById(tfCustomFlowEntity);
        return this.saveOrUpdate(tbProjectExchange);
    }

    /**
     * 项目变更计划申请列表
     *
     * @param iPage
     * @param serviceId
     * @param createUserId
     * @param type
     * @param customType
     * @param flowStatus
     * @param customName
     * @param exchangeNo
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public IPage<Map> list(IPage<Map> iPage, Long serviceId, Long createUserId, DelayWarnTypeEnum type, ProductTypeEnum customType, ProjectExchangeFlowStatusEnum flowStatus, String customName, String exchangeNo, Long startTime, Long endTime) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        createUserId = Objects.isNull(createUserId) ? sysUser.getId() : createUserId;
        return this.baseMapper.list(iPage, serviceId, createUserId, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(customType) ? customType.name() : null, Objects.nonNull(flowStatus) ? flowStatus.name() : null, customName, exchangeNo, startTime, endTime);
    }
}
